import json
from pathlib import Path


class Config:

    def __init__(self, config_file:Path = None):
        if config_file is not None:
            self.config_file = config_file
        else:
            # Default config file
            self.config_file = Path.home() / '.config' / 'diario' / 'config.json'
        # Crea la cartella principale se non esiste
        self.config_file.parent.mkdir(parents=True, exist_ok=True)
        if not self.config_file.exists():
            # Crea un file di configurazione di default se il file non esiste
            base_dir = Path.home() / ".local" / "share" / "diario"
            base_dir.mkdir(parents=True, exist_ok=True)
            default_config = {}
            default_config["path"] = str(base_dir)
            default_config["locale"] = "en_US"
            default_config["editor"] = "nano"
            self.config_file.write_text(
                json.dumps(default_config, indent=2, ensure_ascii=False),
                encoding="utf-8",
            )
        self.settings = json.load(self.config_file.open())


# Questa riga serve perchè al momento
# di eseguire il codice ancora non
# sa cosa sia il Diary.Contents
# restituito da Diary.get_day()
from __future__ import annotations

import datetime
import locale
from dataclasses import dataclass
from pathlib import Path
from typing import Iterator
from diario.config import Config
from diario.templates import Templates
from diario.diaryfiles import DiaryFiles


class Diary:

    class Contents:

        def __init__(self):
            self.index:Path|None = None
            self.attachements:list[Path] = []

        def __iter__(self) -> Iterator[Path]:
            if self.index is not None:
                yield self.index
            yield from self.attachements

        def __len__(self) -> int:
            return 1 + len(self.attachements)

    @dataclass
    class ContentsList:
        daily: Diary.Contents | None
        weekly: Diary.Contents | None
        quarterly: Diary.Contents | None
        monthly: Diary.Contents | None
        yearly: Diary.Contents | None

    def __init__(self, config_file:Path = None):
        self.config = Config(config_file=config_file)
        if self.config.settings.get("locale") is not None:
            try:
                locale.setlocale(locale.LC_TIME, self.config.settings.get("locale") + ".UTF-8")
            except locale.Error:
                # This should work on Debian and other Linux forks
                try:
                    locale.setlocale(locale.LC_TIME, self.config.settings.get("locale") + ".UTF8")
                except locale.Error:
                    print(f"WARNING: Invalid locale: {self.config.settings.get('locale')}")

        #self.today = datetime.date.today()
        base_dir = Path(self.config.settings["path"])
        base_dir.mkdir(parents=True, exist_ok=True)
        if base_dir.is_dir() and base_dir.exists():
            self.files = DiaryFiles(base_dir=base_dir)
            self.templates = Templates(path=base_dir / "templates", files=self.files)
        else:
            raise ValueError(f"Invalid base directory: {base_dir}")

    def _build_contents(self, file_path: Path, template_text: str) -> Diary.Contents:

        # Crea la cartella se non esiste
        file_path.mkdir(parents=True, exist_ok=True)

        contents = Diary.Contents()
        contents.index = file_path / "_index.md"

        # Se il file indice non esiste lo crea
        if not contents.index.exists():
            contents.index.write_text(template_text, encoding="utf-8")

        # Aggiunge tutti gli altri file
        # nella cartella come allegati
        for file in file_path.iterdir():
            if file.is_file() and file.name != "_index.md":
                contents.attachements.append(file)

        return contents

    def get(self, day:datetime.date = None) -> Diary.ContentsList:

        day = self.today if day is None else day

        return Diary.ContentsList(
            yearly=self.get_single(day, view="yearly"),
            quarterly=self.get_single(day, view="quarterly"),
            monthly=self.get_single(day, view="monthly"),
            weekly=self.get_single(day, view="weekly"),
            daily=self.get_single(day, view="daily"),
        )

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def get_single(self, day:datetime.date = None, view:str="daily") -> Diary.Contents:

        day = self.today if day is None else day

        file_path = self.files.get(day, view=view)
        if view == "daily":
            template = self.templates.get_daily(day)
        elif view == "weekly":
            template = self.templates.get_weekly(day)
        elif view == "monthly":
            template = self.templates.get_monthly(day)
        elif view == "quarterly":
            template = self.templates.get_quarterly(day)
        elif view == "yearly":
            template = self.templates.get_yearly(day)
        else:
            raise ValueError(f"Invalid view: {view}")

        return self._build_contents(file_path=file_path, template_text=template)


import datetime
import subprocess
import sys

from diario.diary import Diary


def info_quit(code:int=1) -> None:
    msg = "Uso: diario [--tomorrow | --yesterday | --populate | GG/MM/AAAA | GG-MM-AAAA]"
    if code == 0:
        print(msg)
        raise SystemExit(0)
    raise SystemExit(msg)

def _parse_date(s: str) -> datetime.date:
    try:
        return datetime.datetime.strptime(s, "%d/%m/%Y").date()
    except ValueError as e:
        try:
            return datetime.datetime.strptime(s, "%d-%m-%Y").date()
        except ValueError as e:
            raise SystemExit(f"Data non valida: '{s}'. Formato atteso: GG/MM/AAAA o GG-MM-AAAA (es. 20/12/2026 o 20-12-2026).") from e

def main() -> None:

    # diario            -> oggi
    # diario GG/MM/AAAA -> giorno specifico

    diario = Diary()
    today = datetime.date.today()
    day = None

    args = sys.argv[1:]
    if len(args) == 0:
        day = today
    elif len(args) == 1:
        arg = args[0].lower().strip()
        if arg == "--tomorrow":
            day = today + datetime.timedelta(days=1)
        elif arg == "--yesterday":
            day = today - datetime.timedelta(days=1)
        elif arg == "--populate":
            day = today + datetime.timedelta(days=365)
            for i in range(366):
                collezione = diario.get(day=day - datetime.timedelta(days=i))
            quit(0)
        elif arg == "--help" or arg == "-h":
            info_quit(0)
        else:
            day = _parse_date(args[0])
    else:
        info_quit()

    collezione = diario.get(day)
    #print(collezione.daily.index)
    #print(collezione.weekly.index)
    #print(collezione.monthly.index)
    #print(collezione.quarterly.index)
    #print(collezione.yearly.index)

    command = []
    editor = diario.config.settings.get("editor", "nano")
    if type(editor) is list:
        command.extend(editor)
    else:
        command.append(editor)
    command.append(str(collezione.weekly.index))
    command.append(str(collezione.daily.index))

    subprocess.run(command, check=False)

if __name__ == "__main__":
    main()


import datetime
import os
from pathlib import Path


class DiaryFiles:

    def __init__(self, base_dir: Path, folder_name: str = "diary"):
        self.base_dir = base_dir
        self.diary_dir = base_dir / folder_name
        self.daily_dir = self.diary_dir / "daily"
        self.weekly_dir = self.diary_dir / "weekly"
        self.monthly_dir = self.diary_dir / "monthly"
        self.quarterly_dir = self.diary_dir / "quarterly"
        self.yearly_dir = self.diary_dir / "yearly"

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def _get_day_path(self, day:datetime.date) -> Path:
        # Struttura: {folder_name}/daily/YYYY/MM/DD/
        return self.daily_dir / f"{day:%Y}" / f"{day:%m}" / f"{day:%d}"

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def _get_week_path(self, day: datetime.date) -> Path:
        # Struttura: {folder_name}/weekly/YYYY/VV/
        iso_year, iso_week, _ = day.isocalendar()
        return self.weekly_dir / f"{iso_year}" / f"{iso_week:02d}"

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def _get_month_path(self, day: datetime.date) -> Path:
        # Struttura: {folder_name}/monthly/YYYY/MM/
        return self.monthly_dir / f"{day:%Y}" / f"{day:%m}"

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def _get_quarter_path(self, day: datetime.date) -> Path:
        # Struttura: {folder_name}/quarterly/YYYY/QQ/
        quarter = str((day.month - 1) // 3 + 1).zfill(2)
        return self.quarterly_dir / f"{day:%Y}" / quarter

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def _get_year_path(self, day: datetime.date) -> Path:
        # Struttura: {folder_name}/yearly/YYYY/
        return self.yearly_dir / f"{day:%Y}"

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def get(self, day: datetime.date, view: str = "daily") -> Path:
        if view == "daily":
            return self._get_day_path(day)
        elif view == "weekly":
            return self._get_week_path(day)
        elif view == "monthly":
            return self._get_month_path(day)
        elif view == "quarterly":
            return self._get_quarter_path(day)
        elif view == "yearly":
            return self._get_year_path(day)
        elif view == "yesterday":
            return self._get_day_path(day - datetime.timedelta(days=1))
        elif view == "tomorrow":
            return self._get_day_path(day + datetime.timedelta(days=1))
        else:
            raise ValueError(f"Invalid view: {view}")

    # -=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_-=-_

    def get_relative_path(self, day: datetime.date, source_view: str, destination_view: str) -> str:

        # Origin
        source_path = self.get(day, view=source_view)
        destination_path = self.get(day, view=destination_view) / "_index.md"

        relative = os.path.relpath(destination_path, source_path)
        return relative





import datetime
from dataclasses import dataclass
from pathlib import Path
import shutil

from diario.datevariables import DateVariables
from diario.diaryfiles import DiaryFiles


class Templates:

    def __init__(self, path: Path, files:DiaryFiles):
        self.path = path
        self.path.mkdir(parents=True, exist_ok=True)
        self.files = files
        self.dates = DateVariables(files=self.files)

    def _return_template(self, filename:str, day:datetime.date):
        if not self.path.exists():
            self.path.mkdir(parents=True, exist_ok=True)
        temp = self.path / filename
        if not temp.exists():
            origin = Path(__file__).parent / "resources" / "templates" / filename
            shutil.copy2(origin, temp)

        if temp.is_file() and temp.exists():
            self.dates.render(day)
            text = temp.read_text(encoding="utf-8")
            return self.dates.replace(text)
        else:
            return ""

    def get_daily(self, day:datetime.date) -> str:
        return self._return_template(filename="daily.md", day=day)

    def get_weekly(self, day:datetime.date) -> str:
        return self._return_template(filename="weekly.md", day=day)

    def get_monthly(self, day:datetime.date) -> str:
        return self._return_template(filename="monthly.md", day=day)

    def get_quarterly(self, day:datetime.date) -> str:
        return self._return_template(filename="quarterly.md", day=day)

    def get_yearly(self, day:datetime.date) -> str:
        return self._return_template(filename="yearly.md", day=day)


import datetime
import re

from diario.diaryfiles import DiaryFiles


class DateVariables:

    _pattern: re.Pattern = re.compile(r"\{\{\s*([a-zA-Z_][a-zA-Z0-9_]*)\s*\}\}")

    def __init__(self, files:DiaryFiles) -> None:
        self._day: datetime.date|None = None
        self._files = files
        self.day_name: str = ""
        self.day_name_short: str = ""
        self.day_name_short_2: str = ""
        self.day: str = ""
        self.month: str = ""
        self.month_name: str = ""
        self.month_name_short: str = ""
        self.year: str = ""
        self.year_short: str = ""
        self.quarter: str = ""
        self.yesterday_day_name: str = ""
        self.yesterday_day_name_short: str = ""
        self.yesterday_day_name_short_2: str = ""
        self.yesterday_day: str = ""
        self.yesterday_month: str = ""
        self.yesterday_month_name: str = ""
        self.yesterday_month_name_short: str = ""
        self.yesterday_year: str = ""
        self.yesterday_year_short: str = ""
        self.tomorrow_day_name: str = ""
        self.tomorrow_day_name_short: str = ""
        self.tomorrow_day_name_short_2: str = ""
        self.tomorrow_day: str = ""
        self.tomorrow_month: str = ""
        self.tomorrow_month_name: str = ""
        self.tomorrow_month_name_short: str = ""
        self.tomorrow_year: str = ""
        self.tomorrow_year_short: str = ""
        self.monday_day_name: str = ""
        self.monday_day_name_short: str = ""
        self.monday_day_name_short_2: str = ""
        self.monday_day: str = ""
        self.monday_month: str = ""
        self.monday_month_name: str = ""
        self.monday_month_name_short: str = ""
        self.monday_year: str = ""
        self.monday_year_short: str = ""
        self.sunday_day_name: str = ""
        self.sunday_day_name_short: str = ""
        self.sunday_day_name_short_2: str = ""
        self.sunday_day: str = ""
        self.sunday_month: str = ""
        self.sunday_month_name: str = ""
        self.sunday_month_name_short: str = ""
        self.sunday_year: str = ""
        self.sunday_year_short: str = ""
        self.daily_url: str = ""
        self.weekly_url: str = ""
        self.monthly_url: str = ""
        self.quarterly_url: str = ""
        self.yearly_url: str = ""
        self.today_url: str = ""
        self.yesterday_url: str = ""
        self.tomorrow_url: str = ""
        self.monday_url: str = ""
        self.tuesday_url: str = ""
        self.wednesday_url: str = ""
        self.thursday_url: str = ""
        self.friday_url: str = ""
        self.saturday_url: str = ""
        self.sunday_url: str = ""

    @staticmethod
    def calculate_yesterday(day:datetime.date):
        return day - datetime.timedelta(days=1)

    @staticmethod
    def calculate_tomorrow(day:datetime.date):
        return day + datetime.timedelta(days=1)

    @staticmethod
    def calculate_monday(day:datetime.date):
        return day - datetime.timedelta(days=day.isoweekday() - 1)

    @staticmethod
    def calculate_sunday(day:datetime.date):
        return day + datetime.timedelta(days=7 - day.isoweekday())

    @staticmethod
    def _assign_values(day:datetime.date):
        return (day.strftime("%A").title(),
                day.strftime("%a").title(),
                day.strftime("%a").title()[:2],
                day.strftime("%d"),
                day.strftime("%m"),
                day.strftime("%B").title(),
                day.strftime("%b").title(),
                day.strftime("%Y"),
                day.strftime("%y"))

    def _render(self, day:datetime.date) -> None:
        # Calcola i giorni
        yesterday = self.calculate_yesterday(day)
        tomorrow = self.calculate_tomorrow(day)
        monday = self.calculate_monday(day)
        sunday = self.calculate_sunday(day)

        # Giorno richiesto
        (self.day_name, self.day_name_short,
         self.day_name_short_2, self.day, self.month,
         self.month_name, self.month_name_short,
         self.year, self.year_short) = self._assign_values(day)

        # Ieri
        (self.yesterday_day_name, self.yesterday_day_name_short,
         self.yesterday_day_name_short_2, self.yesterday_day, self.yesterday_month,
         self.yesterday_month_name, self.yesterday_month_name_short,
         self.yesterday_year, self.yesterday_year_short) = self._assign_values(yesterday)

        # Domani
        (self.tomorrow_day_name, self.tomorrow_day_name_short,
         self.tomorrow_day_name_short_2, self.tomorrow_day, self.tomorrow_month,
         self.tomorrow_month_name, self.tomorrow_month_name_short,
         self.tomorrow_year, self.tomorrow_year_short) = self._assign_values(tomorrow)

        # Lunedì
        (self.monday_day_name, self.monday_day_name_short,
         self.monday_day_name_short_2, self.monday_day, self.monday_month,
         self.monday_month_name, self.monday_month_name_short,
         self.monday_year, self.monday_year_short) = self._assign_values(monday)

        # Martedì
        (self.tuesday_day_name, self.tuesday_day_name_short,
         self.tuesday_day_name_short_2, self.tuesday_day, self.tuesday_month,
         self.tuesday_month_name, self.tuesday_month_name_short,
         self.tuesday_year, self.tuesday_year_short) = self._assign_values(monday + datetime.timedelta(days=1))

        # Mercoledì
        (self.wednesday_day_name, self.wednesday_day_name_short,
         self.wednesday_day_name_short_2, self.wednesday_day, self.wednesday_month,
         self.wednesday_month_name, self.wednesday_month_name_short,
         self.wednesday_year, self.wednesday_year_short) = self._assign_values(monday + datetime.timedelta(days=2))

        # Giovedì
        (self.thursday_day_name, self.thursday_day_name_short,
         self.thursday_day_name_short_2, self.thursday_day, self.thursday_month,
         self.thursday_month_name, self.thursday_month_name_short,
         self.thursday_year, self.thursday_year_short) = self._assign_values(monday + datetime.timedelta(days=3))

        # Venerdì
        (self.friday_day_name, self.friday_day_name_short,
         self.friday_day_name_short_2, self.friday_day, self.friday_month,
         self.friday_month_name, self.friday_month_name_short,
         self.friday_year, self.friday_year_short) = self._assign_values(monday + datetime.timedelta(days=4))

        # Sabato
        (self.saturday_day_name, self.saturday_day_name_short,
         self.saturday_day_name_short_2, self.saturday_day, self.saturday_month,
         self.saturday_month_name, self.saturday_month_name_short,
         self.saturday_year, self.saturday_year_short) = self._assign_values(monday + datetime.timedelta(days=5))

        # Domenica
        (self.sunday_day_name, self.sunday_day_name_short,
         self.sunday_day_name_short_2, self.sunday_day, self.sunday_month,
         self.sunday_month_name, self.sunday_month_name_short,
         self.sunday_year, self.sunday_year_short) = self._assign_values(sunday)

        # Trimestre
        self.quarter = "Q" + str((day.month - 1) // 3 + 1)

        # Percorsi ai files
        self.yearly_url = self._files.get_relative_path(day, source_view="quarterly", destination_view="yearly")
        self.quarterly_url = self._files.get_relative_path(day, source_view="monthly", destination_view="quarterly")
        self.monthly_url = self._files.get_relative_path(day, source_view="weekly", destination_view="monthly")
        self.weekly_url = self._files.get_relative_path(day, source_view="daily", destination_view="weekly")
        self.daily_url = str(self._files.get(day, view="daily"))

        self.yesterday_url = self._files.get_relative_path(day, source_view="daily", destination_view="yesterday")
        self.today_url = self.daily_url
        self.tomorrow_url = self._files.get_relative_path(day, source_view="daily", destination_view="tomorrow")

        temp = monday
        self.monday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp += datetime.timedelta(days=1)
        self.tuesday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp += datetime.timedelta(days=1)
        self.wednesday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp += datetime.timedelta(days=1)
        self.thursday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp += datetime.timedelta(days=1)
        self.friday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp += datetime.timedelta(days=1)
        self.saturday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")
        temp = sunday
        self.sunday_url = self._files.get_relative_path(temp, source_view="weekly", destination_view="daily")

    def render(self, day:datetime.date):
        if self._day != day:
            self._render(day)
            self._day = day

    def _replace(self, match: re.Match) -> str:
        name = match.group(1)

        # escludi attributi interni o inesistenti
        if name.startswith("_"):
            return match.group(0)

        if not hasattr(self, name):
            return match.group(0)

        value = getattr(self, name)
        return str(value)

    def replace(self, text:str) -> str:
        return DateVariables._pattern.sub(self._replace, text)